package com.rrd.spring.event.scanner;

import java.lang.reflect.Method;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ApplicationEvent;

import com.rrd.spring.event.EventHandler;
import com.rrd.spring.event.router.EventRouter;
import com.rrd.spring.event.router.Invoker;
import com.rrd.spring.event.router.ReflectionInvoker;

/**
 * A {@link BeanFactoryPostProcessor} that scans every instantiated bean for {@link EventHandler} annotations and registers a router
 * that routes concrete events to the event handling methods.
 * 
 * @see EventHandler
 * @author eugene_rachitskiy
 */
public class EventHandlerBeanFactoryPostProcessor implements BeanFactoryPostProcessor {
	private final EventRouter router = new EventRouter();
	private static final Logger logger = LoggerFactory
			.getLogger(EventHandlerBeanFactoryPostProcessor.class);

	public void postProcessBeanFactory(ConfigurableListableBeanFactory cbf)
			throws BeansException {
		cbf.registerSingleton(EventRouter.class.getName(), router);
		cbf.addBeanPostProcessor(new BeanPostProcessor(){			
			public Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException {
				return bean;
			}

			public Object postProcessAfterInitialization(Object bean,String beanName) throws BeansException { 
				if(bean==null){
					return null;
				}
				Class<?> beanClass=bean.getClass();
				for(final Method method:beanClass.getDeclaredMethods()){
					EventHandler eventHandlerAnnotation = method.getAnnotation(EventHandler.class);
					if(eventHandlerAnnotation!=null){												
						if(method.getParameterTypes()==null||method.getParameterTypes().length!=1||!ApplicationEvent.class.isAssignableFrom(method.getParameterTypes()[0])){
							logger.error("Bean "+beanName+" of type "+beanClass.getSimpleName()+" is annotated with "+EventHandler.class.getSimpleName()+" but does not have a single concrete ApplicationContextEvent parameter.");
						}else{							
							Class<?> eventType = method.getParameterTypes()[0];
							Invoker invoker = new ReflectionInvoker(bean, method);
							router.getInvokerMap().get(eventType).add(invoker);
						}
					}
				}
				return bean;
			}			
		});	
	}
}